﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using PT.Scheduler;

namespace PlanetTogether.PlanetTogether.NoAnchorDrift
{
    class ATPManager
    {
        private readonly Dictionary<BaseId, Dictionary<BaseId, ATPInfo>> m_warehouseDictionary; 
        public ATPManager()
        {
            m_warehouseDictionary = new Dictionary<BaseId, Dictionary<BaseId, ATPInfo>>();
        }

        public void Clear()
        {
            m_warehouseDictionary.Clear();
        }

        public void AddATPInfo(ATPInfo a_atpInfo)
        {
            Dictionary<BaseId, ATPInfo> m_adjustmentsDictionary;
            if (m_warehouseDictionary.TryGetValue(a_atpInfo.WarehouseId, out m_adjustmentsDictionary))
            {
                if (!m_adjustmentsDictionary.ContainsKey(a_atpInfo.ItemId))
                {
                    m_adjustmentsDictionary.Add(a_atpInfo.ItemId, a_atpInfo);
                }
            }
            else
            {
                //No inventories have been added for this warehouse yet.
                m_adjustmentsDictionary = new Dictionary<BaseId, ATPInfo>();
                m_adjustmentsDictionary.Add(a_atpInfo.ItemId, a_atpInfo);
                m_warehouseDictionary.Add(a_atpInfo.WarehouseId, m_adjustmentsDictionary);
            }
        }

        public bool IsManaged(BaseId a_warehouseId, BaseId a_itemId)
        {
            Dictionary<BaseId, ATPInfo> adjustmentsDictionary;
            if (m_warehouseDictionary.TryGetValue(a_warehouseId, out adjustmentsDictionary))
            {
                if (adjustmentsDictionary.ContainsKey(a_itemId))
                {
                    return true;
                }
            }

            return false;
        }

        //TODO: Test with Veridiam Scenario. See why material shortage appear when re-adding an unscheduled job.
        /// <summary>
        /// Returns whether there is enough material to use the specified ammount at the specified time
        /// </summary>
        /// <returns></returns>
        public long GetScheduleDelayWithATP(BaseId a_warehouseId, BaseId a_itemId, long a_clock, long a_requiredTime, decimal a_requiredQty, BaseIdObject a_reason)
        {
            //Go through all adjustments within lead time. Assume the requiredQty can schedule at the specified time.
            //If the min inventory level ever goes negative, return false.
            Dictionary<BaseId, ATPInfo> adjustmentsDictionary;
            if (m_warehouseDictionary.TryGetValue(a_warehouseId, out adjustmentsDictionary))
            {
                ATPInfo atpInfo;
                if (adjustmentsDictionary.TryGetValue(a_itemId, out atpInfo))
                {
                    decimal projectedInv = atpInfo.OnHandQty;
                    for (int i = 0; i < atpInfo.Adjustments.Count; i++)
                    {
                        Adjustment adj = atpInfo.Adjustments[i];
                        if (adj.Time.Ticks >= a_clock + atpInfo.LeadTime.Ticks)
                        {
                            //We are past the lead time, so the remaining adjustments don't matter.
                            //Other jobs will be able to schedule by getting material from lead time if needed.
                            return 0;
                        }
                        if (adj.ReasonId == a_reason.Id)
                        {
                            //Don't count this adjustment because the object attempting to schedule is the same one that caused this adjustment
                            continue;
                        }
                        if (adj.Time.Ticks >= a_requiredTime)
                        {
                            //Insert the requiredQty here.
                            projectedInv -= a_requiredQty;
                            if (projectedInv < 0)
                            {
                                //Check if the next adjustment is at the same time. This could increase available qty
                                if (i < atpInfo.Adjustments.Count - 1 && atpInfo.Adjustments[i + 1].Time == adj.Time)
                                {
                                    continue;
                                }
                                return TryAfterAdjustmentTicks(adj);
                            }
                            a_requiredQty = 0;
                        }
                        projectedInv += adj.ChangeQty;
                        if (projectedInv < 0)
                        {
                            //Check if the next adjustment is at the same time. This could increase available qty
                            if (i < atpInfo.Adjustments.Count - 1 && atpInfo.Adjustments[i + 1].Time == adj.Time)
                            {
                                continue;
                            }
                            return TryAfterAdjustmentTicks(adj);
                        }
                    }
                }
                else
                {
                    //No ATP information was added for this item
                }
            }
            else
            {
                //no ATP information was added for this warehouse.
            }

            return 0;
        }

        /// <summary>
        /// The time after an adjustment
        /// </summary>
        /// <param name="a_adj"></param>
        /// <returns></returns>
        long TryAfterAdjustmentTicks(Adjustment a_adj)
        {
            return a_adj.Time.Ticks + 1;
        }

        public void InventoryUpdate(BaseId a_warehouseId, BaseId a_itemId, DateTime a_time, decimal a_adjustmentQty, InternalOperation a_reason)
        {
            //Update the 
            Dictionary<BaseId, ATPInfo> adjustmentsDictionary;
            if (m_warehouseDictionary.TryGetValue(a_warehouseId, out adjustmentsDictionary))
            {
                ATPInfo atpInfo;
                if (adjustmentsDictionary.TryGetValue(a_itemId, out atpInfo))
                {
                    //atpInfo.Adjustments.Add(new Adjustment(a_time, a_adjustmentQty, a_reason.Id));
                    //Testing
                    atpInfo.Adjustments.Add(new Adjustment(a_time, a_adjustmentQty, a_reason));
                }
            }
        }
    }
}
