﻿using System.Collections.Generic;

using PT.Scheduler;
using PT.Scheduler.Simulation.Customizations;
using PT.Transmissions;

namespace PlanetTogether.PlanetTogether.NoAnchorDrift
{
    /// <summary>
    /// This customization will use data stored in the activities to create CIP offline capacity intervals on the schedule.
    /// </summary>
    public class PostSimulation : PostSimStageCustomization
    {
        NoAnchorDrift m_noAnchorDriftCust = null;

        protected override void PostSimStageSetup(ScenarioDetail.SimulationType a_simType, ScenarioBaseT a_t, ScenarioDetail a_sd, PostSimStageChangeHelper a_changeHelper, int a_currentSimStageIdx, int a_lastSimStageIdx)
        {
            if (m_noAnchorDriftCust == null)
            {
                m_noAnchorDriftCust = a_sd.CustomizationInstances.SchedulabilityCustomization as NoAnchorDrift;
                if (m_noAnchorDriftCust == null) return;
            }
            PostSimStageSetup(a_simType, a_t, a_sd, a_changeHelper, a_lastSimStageIdx, a_lastSimStageIdx, m_noAnchorDriftCust);
        }

        public void PostSimStageSetup(ScenarioDetail.SimulationType a_simType, ScenarioBaseT a_t, ScenarioDetail a_sd, PostSimStageChangeHelper a_changeHelper, int a_currentSimStageIdx, int a_lastSimStageIdx, NoAnchorDrift a_noAnchorDriftCust)
        {
            if (a_noAnchorDriftCust != null)
            {
                //Store all material adjustments after every simulation
                a_noAnchorDriftCust.PostSimulationInventoryStorageClear();
                foreach (Warehouse wh in a_sd.WarehouseManager)
                {
                    foreach (InventoryManager.ItemInventory itemInventory in wh.Inventories)
                    {
                        a_noAnchorDriftCust.PostSimulationInventoryStorage(itemInventory.Inventory);
                    }
                }

                //Don't update jobs if this was not a CTP simulation
                if (!a_noAnchorDriftCust.IsCTPSimulation())
                {
                    return;
                }

                foreach (Job job in a_sd.JobManager)
                {
                    List<BaseOperation> baseOperations = job.GetOperations();
                    foreach (BaseOperation op in baseOperations)
                    {
                        if (op.Scheduled && op.OnHold)
                        {
                            long originalHold = a_noAnchorDriftCust.GetOriginalHoldDate(op);
                            BaseOperationValueChanges opChanges = new BaseOperationValueChanges(op);
                            if (originalHold != long.MaxValue)
                            {
                                //The op was originally on hold, restore its hold date
                                opChanges.OnHold = true;
                                opChanges.HoldUntil = originalHold;
                            }
                            else
                            {
                                //The op was not on hold before the sim start
                                opChanges.OnHold = false;
                            }
                            a_changeHelper.SchedulabilityChangeHelper.AddOperationChanger(opChanges);
                        }
                    }
                }
            }
        }

        public override string Name => "PostSimulation";

        public override string Description => "Part of the CTP customization";
    }
}
